/*
 * Copyright (C) 2022 luca0N!
 *
 * This file is part of Static Website Generator (swg).
 *
 * Static Website Generator (swg) is free software: you can redistribute it
 * and/or modify it under the terms of the version 3 of the GNU Lesser General
 * Public License as published by the Free Software Foundation.
 *
 * Static Website Generator (swg) is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Static Website Generator (swg). If not, see
 * <https://www.gnu.org/licenses/>.
 *
 * Contact luca0N! by e-mail: <luca0n [at] luca0n [dot] com>. 
 */

#pragma once

#include <string>

namespace Article {
	struct Metadata {
		char title[128];
		char authors[128];		// Author (or authors) for this article
		char path[256];			// Path to this article

		// Epoch timestamps
		long original_ts;		// Original article timestamp (for "Originally written on [...]")
		long publish_ts;		// (for "Published on [...]")
		long update_ts;			// (for "Last updated on [...]")
		char publish_str[32];		// Override localized publish timestamps

		// Article flags
		bool partially_obsoleted,
		     obsoleted,
		     mature_content;

		char partially_obsoleted_reason[256],
		     obsoleted_reason[256],
		     obsoletion_followup_article[128],

		     mature_content_reason[256];
	};
	class Comparator {
		public:
			static bool comp(Metadata *a, Metadata *b);
			static bool equiv(Metadata *a, Metadata *b);
	};

	void get_metadata(std::string const &path, Metadata *m);
}

